/*
 * Copyright (c) Tony Bybell 2010
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifndef WAVE_ENTRY_H
#define WAVE_ENTRY_H

void entrybox(char *title, int width, char *dflt_text, char *comment, int maxch, GCallback func);

#endif

