
#line 3 "vlex.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 37
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart(yyin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

extern yy_size_t yyleng;

extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	yy_size_t yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;
static yy_size_t yy_n_chars;		/* number of characters read into yy_ch_buf */
yy_size_t yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart (FILE *input_file  );
void yy_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE yy_create_buffer (FILE *file,int size  );
void yy_delete_buffer (YY_BUFFER_STATE b  );
void yy_flush_buffer (YY_BUFFER_STATE b  );
void yypush_buffer_state (YY_BUFFER_STATE new_buffer  );
void yypop_buffer_state (void );

static void yyensure_buffer_stack (void );
static void yy_load_buffer_state (void );
static void yy_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER yy_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE yy_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE yy_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE yy_scan_bytes (yyconst char *bytes,yy_size_t len  );

void *yyalloc (yy_size_t  );
void *yyrealloc (void *,yy_size_t  );
void yyfree (void *  );

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;

typedef int yy_state_type;

extern int yylineno;

int yylineno = 1;

extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	yyleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 157
#define YY_END_OF_BUFFER 158
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[619] =
    {   0,
        0,    0,    0,    0,    0,    0,  158,  156,  152,  153,
      156,  156,  156,  156,  156,  156,  146,  143,  156,  156,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,    4,    6,    5,    5,    5,  138,  139,  152,
        0,  155,  142,    0,    0,    0,    0,  151,  145,    1,
      154,  146,  143,    0,  141,  140,  143,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,   33,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,

      143,   48,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,    4,    3,    2,  138,  142,    0,
      148,    0,  147,    0,  150,    0,  149,  145,  154,  144,
      140,  143,    8,  143,  143,  143,  126,   11,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,   24,
      143,  143,   27,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  128,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,  143,   44,  143,   45,  143,  143,

      143,  143,  143,  143,  143,  143,   60,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,   79,  143,  124,  143,  143,  143,  143,
      143,  143,  143,   92,  143,   94,  148,  147,  150,  149,
      144,  143,  143,  143,  143,  143,  131,   14,  104,   17,
      143,  143,  143,  143,  143,  143,   22,   23,  143,  143,
      143,  143,  143,  143,  143,  132,  143,  143,  143,   30,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,   38,  143,  143,  143,  143,  143,  143,  143,
      143,  143,   41,  143,   43,  143,  143,  143,  143,   51,

      143,  143,  143,  143,  143,   58,  143,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,  143,
       95,   75,   76,   80,   81,  143,  143,  143,  143,  143,
      143,   86,   87,  143,  143,   91,   93,  143,  143,  143,
       10,  143,   15,   16,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,   26,   28,
      143,  143,  143,  143,  143,  143,  143,  143,  143,   35,
       36,  143,  143,   39,  143,  143,  143,  127,  143,  143,
      143,  143,  143,  143,  143,  143,  143,  143,  143,   53,
       54,  143,  143,  143,   57,  143,  143,  143,   63,   64,

       65,  143,  143,  143,  143,   69,  143,  143,  143,  143,
       99,  143,  143,   83,  143,  143,  125,  143,   88,   89,
       90,    7,    9,  143,   12,   13,  105,  143,  143,  143,
      106,  143,  143,  143,  143,  143,  143,  143,  143,  143,
      143,  143,  143,  143,  114,   31,   32,  109,  110,  143,
      143,  143,  143,  143,  143,  143,  143,  143,   40,  134,
      143,  143,   46,   47,   49,  143,  143,  143,  143,   56,
      143,  143,  143,   62,  143,  143,  143,  143,  123,  143,
      143,  143,  143,  143,   82,   84,  143,  143,  143,  143,
       19,  143,   21,   25,  143,  143,  143,  143,  143,  143,

      143,   96,   29,  143,  143,  111,   34,  143,   37,  115,
      116,  143,  130,  143,   42,  143,  143,   52,  143,  143,
      143,  143,   61,  143,  143,  143,  143,  101,  143,   71,
       72,   73,   74,   77,   78,  143,  143,  143,   18,   20,
      143,  143,  143,  143,  143,  143,  100,   97,  113,  112,
      143,  143,  143,  143,  143,   55,  143,   59,   66,   67,
       68,  129,  143,  143,  121,   85,  103,  107,  143,  143,
      136,  143,  143,  143,  143,  143,   50,  135,  143,  143,
       70,  143,  143,  143,  102,  117,  143,  143,  143,  143,
       98,  108,  143,  133,  143,  143,  143,  137,  143,  143,

      143,  143,  143,  122,  143,  143,  143,  118,  143,  143,
      143,  143,  143,  143,  143,  119,  120,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    4,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    5,    1,    6,    1,    1,    7,    1,
        1,    8,    1,    1,    1,    9,   10,   11,   12,   13,
       13,   13,   13,   13,   13,   14,   14,    1,    1,    1,
        1,    1,   15,    1,   16,   17,   16,   18,   16,   16,
       19,   20,   19,   19,   19,   19,   19,   19,   21,   19,
       19,   19,   19,   19,   19,   19,   19,   22,   19,   22,
        1,   23,    1,    1,   24,   25,   26,   27,   28,   29,

       30,   31,   32,   33,   34,   35,   36,   37,   38,   39,
       40,   41,   19,   42,   43,   44,   45,   46,   47,   48,
       49,   50,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[51] =
    {   0,
        1,    2,    3,    4,    1,    5,    1,    6,    7,    6,
        8,    8,    8,    8,    1,    9,    9,    9,   10,   10,
       10,    9,    6,   11,    1,    9,    9,    9,    9,    9,
        9,   10,   10,   10,   10,   10,   10,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,    9,   10,    9
    } ;

static yyconst flex_int16_t yy_base[637] =
    {   0,
        0,    0,   48,   49, 1360, 1359, 1361, 1364,   51, 1364,
     1355,    0,   56, 1349,    0,   52,   52, 1349,    0,    0,
       45,   61,   71,   69,   78,   60,   72,   52,   83,   58,
       95,  104,  107,  109,  110,  129,  165,  130,   84,  111,
      149,  103,    0, 1364, 1347, 1348, 1364,    0, 1364,  124,
     1350, 1364,    0,  165,  183, 1352,  179, 1364,    0, 1364,
        0,  149, 1344,    0,    0,    0,   85,  137,  162,  140,
      156,  160,  169,  177,  193,  116,  198,  202,  199,  203,
      205,  207,  210,  213,   66,  208,  209,  217,  214,  223,
      211,  219,  225,  226,  231,  215,  247,  251,  253,  259,

      260, 1343,  261,  264,  268,  266,  262,  263,  269,  284,
      273,  274,  271,  265,  275,  285,  272,  288,  277,  279,
      305,  286,  313,  280,  319,  292,  318,  320,  324,  321,
      322,  325,  326,  327,    0, 1364, 1364,    0,    0,  341,
      346,  360,  364, 1349,    0,  388,  371,    0,    0, 1341,
        0,  371, 1340,  331,  372,  351, 1339,  370,  377,  378,
      383,  389,  393,  394,  400,  396,  397,  404,  405,  433,
      402,  406,  416,  419,  409,  418,  408,  420,  422,  413,
      424,  434,  445,  441,  447,  444,  458,  451,  450,  453,
      454,  459,  461,  463,  464, 1338,  478,  474,  480,  483,

      482,  485,  488,  487,  489,  490, 1337,  492,  493,  491,
      494,  507,  501,  497,  496,  503,  504,  505,  508,  509,
      511,  527,  519,  539,  528, 1336,  510,  532,  533,  534,
      544,  545,  555, 1335,  546, 1334,  572,  578,    0,  586,
     1333,  551,  561,  557,  547,  592, 1332,  559, 1331, 1330,
      569,  562,  566,  595,  597,  603, 1329, 1328,  605,  604,
      607,  606,  608,  610,  609, 1327,  615,  617,  616, 1326,
      619,  618,  629,  623,  630,  624,  631,  632,  633,  634,
      635,  639, 1325,  644,  645,  655,  643,  647,  648,  656,
      658,  661, 1324,  659, 1323,  674,  663,  675,  677, 1322,

      676,  678,  680,  681,  684,  686,  690,  691,  688,  692,
      693,  695,  697,  698,  699,  701,  709,  710,  714,  704,
     1321, 1320,  713, 1319, 1318,  715,  719,  727,  724,  728,
      730, 1317, 1316,  333,  733, 1315, 1314,  731,  736,  739,
     1313,  755, 1312, 1311,  737,  743,  735,  744,  746,  750,
      751,  753,  759,  762,  764,  769,  767,  772, 1310, 1309,
      770,  771,  780,  774,  779,  787,  786,  773,  795, 1308,
     1307,  775,  799, 1306,  798,  800,  802, 1305,  801,  809,
      810,  803,  804,  811,  813,  817,  814,  818,  821, 1304,
     1303,  822,  823,  825, 1302,  826,  828,  829, 1301, 1300,

      836,  830,  840,  837,  846, 1299,  845,  843,  847,  842,
     1298,  850,  848, 1297,  857,  844, 1296,  858, 1295, 1294,
     1293, 1292, 1291,  869, 1290, 1289, 1288,  871,  873,  875,
     1287,  876,  877,  878,  883,  884,  879,  881,  887,  885,
      889,  886,  890,  888, 1286, 1285, 1284, 1283, 1277,  893,
      896,  899,  895,  901,  903,  909,  907,  917, 1274, 1267,
      911,  912, 1266, 1253, 1246,  920,  925,  922,  927, 1245,
      929,  930,  933, 1241,  934,  937,  938,  939, 1240,  935,
      940,  949,  960,  974, 1239, 1238,  945,  950,  953,  957,
     1237,  941, 1231, 1230,  961,  944,  967,  980,  972,  981,

      982, 1220, 1212,  983,  984, 1209, 1196,  988, 1195, 1194,
     1187,  985, 1172,  990, 1171,  994,  991, 1145,  989,  992,
      993,  995, 1144,  996, 1001, 1002, 1004, 1143, 1007, 1142,
     1140, 1139, 1138, 1137, 1134, 1014, 1015, 1017, 1133, 1131,
     1019, 1020, 1027, 1025, 1028, 1029, 1130, 1129, 1128, 1126,
     1030, 1031, 1032, 1038, 1039, 1123, 1040, 1122, 1120, 1118,
     1117, 1113, 1043, 1041,  564,  563,  560,  558, 1048, 1049,
      552, 1050, 1052, 1053, 1055, 1054,  407,  342, 1056, 1057,
      332, 1058, 1059, 1061,  329,  328, 1065, 1066,  550, 1069,
      283,  278, 1072,  270, 1074, 1076, 1078,  224, 1081, 1089,

     1090, 1091, 1094,  222, 1096, 1100, 1095,  131, 1101, 1103,
     1104, 1105, 1108, 1106, 1111,   70,   59, 1364, 1155, 1166,
     1177, 1180, 1183, 1190, 1201, 1204, 1215, 1226, 1233, 1243,
     1248, 1259, 1262, 1269, 1273, 1280
    } ;

static yyconst flex_int16_t yy_def[637] =
    {   0,
      618,    1,  619,  619,  620,  620,  618,  618,  618,  618,
      621,  622,  618,  618,  623,  618,  618,  624,  625,  626,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  627,  618,  618,  618,  618,  628,  618,  618,
      621,  618,  629,  618,  618,  630,  618,  618,  631,  618,
      632,  618,  624,  633,  625,  634,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,

      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  627,  618,  618,  628,  629,  618,
      618,  618,  618,  630,  635,  618,  618,  631,  632,  636,
      634,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,

      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  618,  618,  635,  618,
      636,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,

      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,

      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,

      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,

      624,  624,  624,  624,  624,  624,  624,  624,  624,  624,
      624,  624,  624,  624,  624,  624,  624,    0,  618,  618,
      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618,  618,  618
    } ;

static yyconst flex_int16_t yy_nxt[1415] =
    {   0,
        8,    9,   10,    9,   11,   12,   13,   14,   15,   16,
       17,   17,   17,   17,    8,   18,   18,   18,   18,   18,
       18,   18,   19,   18,   20,   21,   22,   23,   24,   25,
       26,   27,   28,   29,   30,   18,   31,   32,   33,   34,
       35,   36,   37,   38,   39,   40,   41,   42,   18,   18,
       44,   44,   50,   64,   50,   45,   45,   46,   46,   60,
       64,   61,   62,   62,   62,   62,   64,   64,   64,   64,
       47,   47,   54,   55,   64,   56,   57,   64,   64,   64,
       64,   67,   54,   68,   55,   88,   64,   69,   56,   70,
       71,   64,   64,   64,   72,   57,   75,   91,   79,   85,

       76,   87,   80,   64,   86,   73,   81,  173,   77,   74,
       78,   64,   64,   89,   82,   64,   83,   64,   64,   64,
       92,   90,  124,   84,   64,   50,  125,   50,   93,   95,
      126,  152,   98,   96,   94,  104,   99,   64,   64,   64,
      127,  133,  134,   97,  100,   64,  101,  105,   64,  106,
      102,  107,  161,  103,  108,  121,  109,   64,  110,   62,
       62,   62,   62,  122,   64,  153,  140,  111,   64,  112,
       64,  123,  113,   64,  128,  141,  141,   64,  129,  141,
      146,  130,  131,  155,  142,   64,  141,  156,  132,  147,
      147,  147,  114,  143,  143,  143,  143,  115,  116,  158,

      147,   64,  117,  157,  154,  118,   64,   64,  119,  120,
       64,   64,  141,   64,  141,   64,   64,   64,   64,   64,
      159,   64,   64,   64,  164,   64,  147,   64,  147,  165,
       64,   64,   64,   64,   64,  160,  168,  162,  170,   64,
      163,  166,  172,  191,  184,  167,  174,  175,  176,  169,
      178,  186,  177,  187,  171,   64,  179,  188,  190,   64,
      185,   64,  180,  181,  189,  182,  183,   64,   64,   64,
       64,   64,   64,   64,   64,  192,   64,   64,   64,   64,
       64,   64,   64,   64,  194,   64,   64,   64,   64,  193,
      213,   64,   64,   64,   64,  203,   64,  216,  195,  204,

       64,  196,  197,  198,  199,  200,  205,  201,  202,  206,
      210,  211,  212,   64,  214,  207,  215,  217,  218,  219,
      208,   64,  225,  222,  209,  227,   64,   64,   64,   64,
       64,  220,   64,   64,   64,   64,   64,   64,  223,   64,
       64,   64,  140,  419,  420,  228,  224,  221,  226,  231,
       64,  141,  141,  229,  232,  141,  237,  237,  230,   64,
      237,  142,  141,  233,  243,  235,  234,  237,  236,  237,
      143,  143,  143,  143,  238,  238,  238,  238,   64,   64,
       64,  240,  240,  240,  245,   64,   64,  238,  141,  146,
      141,   64,  240,  237,  240,  237,  242,   64,  147,  147,

      147,   64,   64,  246,   64,   64,  247,  248,   64,  147,
       64,  244,   64,   64,   64,   64,   64,   64,  240,  249,
      240,   64,  256,  251,   64,  253,   64,   64,   64,  255,
       64,  250,   64,  257,  258,  147,  252,  147,  272,  266,
      254,   64,   64,  268,  267,  269,  271,  275,  276,   64,
      274,  270,   64,   64,  273,   64,  277,  279,   64,   64,
      259,   64,   64,  260,  261,  278,   64,   64,  280,   64,
      262,   64,   64,  263,  282,  264,  265,  281,  284,  283,
      285,  289,   64,  287,  288,  286,   64,  291,   64,  293,
       64,   64,  294,   64,  290,   64,   64,   64,   64,   64,

       64,   64,   64,  292,   64,   64,  295,  297,  299,   64,
      296,   64,   64,   64,  301,   64,   64,   64,   64,   64,
      298,  307,  308,  303,  300,  302,  306,   64,  305,  304,
      309,  317,  311,  310,  315,   64,   64,  312,  313,  316,
       64,   64,   64,  314,  318,  320,  321,   64,  319,  324,
      325,  330,   64,   64,   64,   64,  322,  323,   64,   64,
       64,  329,  333,   64,  326,   64,   64,   64,   64,   64,
       64,   64,   64,  596,   64,  331,  332,   64,  327,  334,
      328,  335,  237,  237,  336,  341,  237,  337,  238,  238,
      238,  238,  339,  237,  340,  237,  240,  240,  240,  338,

       64,  238,  345,   64,  346,   64,  343,  240,  344,  240,
      347,   64,   64,   64,   64,   64,   64,   64,   64,  237,
      348,  237,  342,   64,   64,   64,   64,   64,  349,  350,
      351,   64,   64,  240,  358,  240,  354,   64,   64,   64,
       64,   64,   64,   64,  352,  355,  360,   64,  353,  356,
      357,   64,   64,   64,  364,   64,   64,  367,  359,  363,
      372,  361,  362,   64,   64,  369,   64,   64,  366,   64,
      373,   64,  365,  374,  378,  368,  370,  371,  375,  377,
      376,  379,   64,   64,   64,   64,   64,  383,   64,   64,
      390,  391,   64,  385,   64,  380,   64,  382,   64,   64,

       64,   64,  381,   64,  388,   64,   64,   64,  392,   64,
      394,  389,   64,  384,  387,  397,  398,   64,   64,  386,
      402,   64,   64,   64,  393,  404,  395,   64,  405,  396,
      399,  401,   64,  411,  400,   64,   64,  406,   64,   64,
      403,   64,  407,   64,   64,   64,  412,   64,  409,  408,
      410,   64,   64,  413,   64,  416,  415,  417,   64,   64,
      414,   64,  421,   64,  424,  425,  426,   64,  427,  418,
       64,  429,   64,  422,  423,   64,  428,   64,   64,   64,
       64,   64,   64,   64,  431,  430,  432,   64,   64,  446,
      447,  434,  437,  433,   64,   64,  439,  435,  440,  442,

      436,  450,  438,   64,  443,  444,   64,   64,   64,   64,
       64,   64,   64,  452,  441,  445,  448,   64,   64,   64,
      451,   64,   64,  463,  464,   64,   64,  449,  453,   64,
       64,   64,  460,   64,   64,  461,   64,   64,   64,  457,
      454,  455,  456,  466,   64,   64,  458,  459,   64,  467,
       64,   64,   64,   64,   64,   64,   64,  462,   64,  472,
      465,  469,  478,  470,  468,   64,   64,  471,  481,  475,
      473,  476,  474,  477,  479,  480,  485,   64,  482,   64,
      484,   64,  487,   64,   64,   64,   64,   64,  486,   64,
      483,   64,   64,   64,   64,   64,   64,   64,   64,  488,

      492,   64,  490,   64,   64,  493,  494,   64,  495,   64,
      496,   64,  489,  497,  500,   64,  491,   64,  499,   64,
       64,  501,  506,  498,  502,   64,  508,  503,   64,  504,
       64,  505,  507,   64,  512,   64,  509,   64,   64,  516,
      515,   64,   64,   64,  510,   64,   64,   64,   64,   64,
      513,  511,   64,   64,  518,  519,  514,   64,   64,  530,
      531,   64,  523,  517,  524,   64,  525,  522,   64,   64,
      532,  533,  521,  520,  536,   64,  526,  527,  540,  537,
       64,  529,   64,  528,  534,  535,  538,  542,   64,   64,
       64,   64,   64,   64,  541,  539,   64,   64,   64,   64,

       64,   64,   64,   64,   64,  545,  559,  560,  543,   64,
       64,  547,   64,  549,  546,   64,  544,  550,  552,  553,
      554,  548,   64,   64,  558,   64,  551,   64,   64,  561,
      556,  563,  564,   64,  555,   64,   64,   64,   64,   64,
       64,  557,  565,  566,  567,  562,   64,   64,   64,   64,
      568,   64,  570,  569,  571,  574,   64,   64,   64,  573,
       64,   64,   64,   64,   64,   64,   64,   64,  578,   64,
      576,  572,  580,   64,   64,  575,  579,   64,  581,  577,
       64,  588,   64,  584,   64,  589,   64,  582,  592,   64,
      586,  587,  583,  590,  594,  595,  591,   64,   64,   64,

      585,  598,   64,   64,   64,  597,  593,  601,   64,   64,
      599,   64,   64,   64,   64,  600,   64,  602,  604,   64,
      605,   64,  606,  607,  608,   64,   64,  603,   64,  609,
       64,   64,  612,  613,   64,  615,   64,   64,   64,   64,
      610,   64,   64,  614,  611,   64,   64,   64,   64,  616,
       64,   64,   64,   64,  617,   43,   43,   43,   43,   43,
       43,   43,   43,   43,   43,   43,   48,   48,   48,   48,
       48,   48,   48,   48,   48,   48,   48,   51,   51,   64,
       64,   51,   51,   51,   51,   51,   51,   51,   53,   53,
       53,   59,   59,   59,   63,   64,   63,   63,   63,   63,

       63,   65,   64,   64,   64,   65,   65,   65,   65,   65,
       65,   65,   66,   66,   66,  135,  135,   64,  135,  135,
       64,  135,  135,  135,  135,  135,  138,  138,   64,  138,
      138,  138,  138,  138,  138,  138,  138,  139,   64,   64,
      139,  139,  139,  139,  145,   64,   64,   64,   64,   64,
      145,  145,  148,   64,   64,  148,  148,  148,  148,  149,
      149,   64,  149,  149,  149,  149,  149,  149,  149,  149,
      150,  150,  150,  151,   64,   64,  151,  151,  151,  151,
      239,  239,   64,  239,  241,   64,  241,  241,  241,  241,
      241,   64,   64,   64,   64,   64,   64,   64,   64,   64,

       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
      144,   64,   64,  144,   52,  137,  136,   64,   58,   52,
      618,   49,   49,    7,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,

      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618
    } ;

static yyconst flex_int16_t yy_chk[1415] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        3,    4,    9,   21,    9,    3,    4,    3,    4,   16,
       28,   16,   17,   17,   17,   17,   30,  617,   26,   22,
        3,    4,   13,   13,   85,   13,   13,   24,  616,   23,
       27,   21,   13,   21,   13,   28,   25,   21,   13,   21,
       22,   29,   39,   67,   22,   13,   23,   30,   24,   26,

       23,   27,   24,   31,   26,   22,   25,   85,   23,   22,
       23,   42,   32,   29,   25,   33,   25,   34,   35,   40,
       31,   29,   39,   25,   76,   50,   39,   50,   31,   32,
       39,   67,   33,   32,   31,   35,   33,   36,   38,  608,
       40,   42,   42,   32,   33,   68,   33,   35,   70,   35,
       34,   35,   76,   34,   35,   38,   36,   41,   36,   62,
       62,   62,   62,   38,   71,   68,   54,   36,   72,   36,
       69,   38,   36,   37,   41,   54,   54,   73,   41,   54,
       57,   41,   41,   70,   55,   74,   54,   71,   41,   57,
       57,   57,   37,   55,   55,   55,   55,   37,   37,   73,

       57,   75,   37,   72,   69,   37,   77,   79,   37,   37,
       78,   80,   54,   81,   54,   82,   86,   87,   83,   91,
       74,   84,   89,   96,   79,   88,   57,   92,   57,   79,
      604,   90,  598,   93,   94,   75,   81,   77,   83,   95,
       78,   79,   84,   96,   91,   80,   86,   87,   88,   82,
       90,   93,   89,   94,   83,   97,   90,   94,   95,   98,
       92,   99,   90,   90,   94,   90,   90,  100,  101,  103,
      107,  108,  104,  114,  106,   97,  105,  109,  594,  113,
      117,  111,  112,  115,   99,  119,  592,  120,  124,   98,
      114,  591,  110,  116,  122,  107,  118,  117,  100,  108,

      126,  101,  101,  101,  103,  104,  109,  105,  106,  110,
      111,  112,  113,  121,  115,  110,  116,  118,  119,  120,
      110,  123,  124,  122,  110,  126,  127,  125,  128,  130,
      131,  121,  129,  132,  133,  134,  586,  585,  123,  154,
      581,  334,  140,  334,  334,  127,  123,  121,  125,  129,
      578,  140,  140,  128,  130,  140,  141,  141,  128,  156,
      141,  142,  140,  131,  154,  133,  132,  141,  134,  141,
      142,  142,  142,  142,  143,  143,  143,  143,  158,  152,
      155,  147,  147,  147,  156,  159,  160,  143,  140,  146,
      140,  161,  147,  141,  147,  141,  152,  162,  146,  146,

      146,  163,  164,  158,  166,  167,  159,  160,  165,  146,
      171,  155,  168,  169,  172,  577,  177,  175,  147,  161,
      147,  180,  167,  163,  173,  165,  176,  174,  178,  166,
      179,  162,  181,  168,  169,  146,  164,  146,  175,  171,
      165,  170,  182,  173,  172,  173,  174,  177,  178,  184,
      176,  173,  186,  183,  175,  185,  178,  180,  189,  188,
      170,  190,  191,  170,  170,  179,  187,  192,  181,  193,
      170,  194,  195,  170,  183,  170,  170,  182,  185,  184,
      186,  189,  198,  187,  188,  186,  197,  191,  199,  193,
      201,  200,  194,  202,  190,  204,  203,  205,  206,  210,

      208,  209,  211,  192,  215,  214,  195,  198,  200,  213,
      197,  216,  217,  218,  202,  212,  219,  220,  227,  221,
      199,  208,  209,  204,  201,  203,  206,  223,  205,  204,
      210,  217,  212,  211,  215,  222,  225,  213,  214,  216,
      228,  229,  230,  214,  218,  220,  221,  224,  219,  224,
      224,  227,  231,  232,  235,  245,  222,  223,  589,  242,
      571,  225,  230,  233,  224,  244,  568,  248,  567,  243,
      252,  566,  565,  589,  253,  228,  229,  251,  224,  231,
      224,  232,  237,  237,  233,  245,  237,  235,  238,  238,
      238,  238,  243,  237,  244,  237,  240,  240,  240,  242,

      246,  238,  251,  254,  252,  255,  248,  240,  248,  240,
      253,  256,  260,  259,  262,  261,  263,  265,  264,  237,
      254,  237,  246,  267,  269,  268,  272,  271,  255,  256,
      259,  274,  276,  240,  265,  240,  261,  273,  275,  277,
      278,  279,  280,  281,  259,  262,  268,  282,  260,  263,
      264,  287,  284,  285,  273,  288,  289,  276,  267,  272,
      281,  269,  271,  286,  290,  278,  291,  294,  275,  292,
      282,  297,  274,  284,  288,  277,  279,  280,  285,  287,
      286,  289,  296,  298,  301,  299,  302,  294,  303,  304,
      303,  303,  305,  297,  306,  290,  309,  292,  307,  308,

      310,  311,  291,  312,  301,  313,  314,  315,  303,  316,
      304,  302,  320,  296,  299,  307,  308,  317,  318,  298,
      312,  323,  319,  326,  303,  314,  305,  327,  315,  306,
      309,  311,  329,  320,  310,  328,  330,  316,  331,  338,
      313,  335,  317,  347,  339,  345,  323,  340,  318,  317,
      319,  346,  348,  326,  349,  329,  328,  330,  350,  351,
      327,  352,  335,  342,  340,  342,  342,  353,  345,  331,
      354,  347,  355,  338,  339,  357,  346,  356,  361,  362,
      358,  368,  364,  372,  349,  348,  350,  365,  363,  365,
      365,  352,  355,  351,  367,  366,  357,  353,  358,  361,

      354,  368,  356,  369,  362,  363,  375,  373,  376,  379,
      377,  382,  383,  372,  358,  364,  366,  380,  381,  384,
      369,  385,  387,  385,  385,  386,  388,  367,  373,  389,
      392,  393,  382,  394,  396,  383,  397,  398,  402,  379,
      375,  376,  377,  387,  401,  404,  380,  381,  403,  388,
      410,  408,  416,  407,  405,  409,  413,  384,  412,  396,
      386,  392,  404,  393,  389,  415,  418,  394,  408,  401,
      397,  402,  398,  403,  405,  407,  413,  424,  409,  428,
      412,  429,  416,  430,  432,  433,  434,  437,  415,  438,
      410,  435,  436,  440,  442,  439,  444,  441,  443,  418,

      430,  450,  428,  453,  451,  432,  433,  452,  434,  454,
      435,  455,  424,  436,  439,  457,  429,  456,  438,  461,
      462,  440,  450,  437,  441,  458,  452,  442,  466,  443,
      468,  444,  451,  467,  456,  469,  453,  471,  472,  462,
      461,  473,  475,  480,  454,  476,  477,  478,  481,  492,
      457,  455,  496,  487,  467,  468,  458,  482,  488,  482,
      482,  489,  473,  466,  475,  490,  476,  472,  483,  495,
      483,  483,  471,  469,  487,  497,  477,  478,  492,  488,
      499,  481,  484,  480,  484,  484,  489,  496,  498,  500,
      501,  504,  505,  512,  495,  490,  508,  519,  514,  517,

      520,  521,  516,  522,  524,  499,  524,  524,  497,  525,
      526,  501,  527,  505,  500,  529,  498,  508,  514,  516,
      517,  504,  536,  537,  522,  538,  512,  541,  542,  525,
      520,  527,  529,  544,  519,  543,  545,  546,  551,  552,
      553,  521,  536,  537,  538,  526,  554,  555,  557,  564,
      541,  563,  543,  542,  544,  551,  569,  570,  572,  546,
      573,  574,  576,  575,  579,  580,  582,  583,  555,  584,
      553,  545,  563,  587,  588,  552,  557,  590,  564,  554,
      593,  576,  595,  572,  596,  579,  597,  569,  583,  599,
      574,  575,  570,  580,  587,  588,  582,  600,  601,  602,

      573,  593,  603,  607,  605,  590,  584,  597,  606,  609,
      595,  610,  611,  612,  614,  596,  613,  599,  601,  615,
      602,  562,  603,  603,  605,  561,  560,  600,  559,  606,
      558,  556,  610,  611,  550,  613,  549,  548,  547,  540,
      607,  539,  535,  612,  609,  534,  533,  532,  531,  614,
      530,  528,  523,  518,  615,  619,  619,  619,  619,  619,
      619,  619,  619,  619,  619,  619,  620,  620,  620,  620,
      620,  620,  620,  620,  620,  620,  620,  621,  621,  515,
      513,  621,  621,  621,  621,  621,  621,  621,  622,  622,
      622,  623,  623,  623,  624,  511,  624,  624,  624,  624,

      624,  625,  510,  509,  507,  625,  625,  625,  625,  625,
      625,  625,  626,  626,  626,  627,  627,  506,  627,  627,
      503,  627,  627,  627,  627,  627,  628,  628,  502,  628,
      628,  628,  628,  628,  628,  628,  628,  629,  494,  493,
      629,  629,  629,  629,  630,  491,  486,  485,  479,  474,
      630,  630,  631,  470,  465,  631,  631,  631,  631,  632,
      632,  464,  632,  632,  632,  632,  632,  632,  632,  632,
      633,  633,  633,  634,  463,  460,  634,  634,  634,  634,
      635,  635,  459,  635,  636,  449,  636,  636,  636,  636,
      636,  448,  447,  446,  445,  431,  427,  426,  425,  423,

      422,  421,  420,  419,  417,  414,  411,  406,  400,  399,
      395,  391,  390,  378,  374,  371,  370,  360,  359,  344,
      343,  341,  337,  336,  333,  332,  325,  324,  322,  321,
      300,  295,  293,  283,  270,  266,  258,  257,  250,  249,
      247,  241,  236,  234,  226,  207,  196,  157,  153,  150,
      144,  102,   63,   56,   51,   46,   45,   18,   14,   11,
        7,    6,    5,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,

      618,  618,  618,  618,  618,  618,  618,  618,  618,  618,
      618,  618,  618,  618
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int yy_flex_debug;
int yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "vlex.l"
/*
 * Copyright (c) Tony Bybell 2006-2011
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */
#line 13 "vlex.l"
        
/*
 * vlex.l
 * 06apr06ajb
 */

#define YY_INPUT(buf, result, max_size) \
   result = v_preproc(buf, max_size);

#include "vlex.h"

int my_yylineno = 0;

static int yywrap(void)
{
return(1);
}

static int v_preproc_initialized = 0;
char *v_preproc_name = NULL;


static int v_preproc(char *buf, int max_size)
{
(void)max_size;

int ch;

if(!v_preproc_initialized)
	{
	yyin = fopen(v_preproc_name, "rb");
	if(!yyin) return(0);
	v_preproc_initialized = 1;
	my_yylineno = 1;	
	}

ch = fgetc(yyin);
if(ch!=EOF)
	{
	buf[0] = ch;
	buf[1] = 0;
	return(1);
	}
	else
	{
	fclose(yyin); yyin = NULL;
	v_preproc_initialized = 0;
	return (0);
	}
}



#line 1031 "vlex.c"

#define INITIAL 0
#define CMNT 1
#define PREPROC 2

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy (void );

int yyget_debug (void );

void yyset_debug (int debug_flag  );

YY_EXTRA_TYPE yyget_extra (void );

void yyset_extra (YY_EXTRA_TYPE user_defined  );

FILE *yyget_in (void );

void yyset_in  (FILE * in_str  );

FILE *yyget_out (void );

void yyset_out  (FILE * out_str  );

yy_size_t yyget_leng (void );

char *yyget_text (void );

int yyget_lineno (void );

void yyset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap (void );
#else
extern int yywrap (void );
#endif
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, yyleng, 1, yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int yylex (void);

#define YY_DECL int yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
#line 69 "vlex.l"

#line 1215 "vlex.c"

	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer(yyin,YY_BUF_SIZE );
		}

		yy_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of yytext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 619 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 1364 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 70 "vlex.l"
{ BEGIN CMNT; return V_CMT; }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 71 "vlex.l"
{ return V_CMT; }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 72 "vlex.l"
{ BEGIN INITIAL; return V_CMT; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 73 "vlex.l"
{ return V_CMT; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 74 "vlex.l"
{ return V_CMT; }
	YY_BREAK
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
#line 75 "vlex.l"
{ my_yylineno++; return V_CMT; }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 77 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 78 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 79 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 80 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 81 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 82 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 83 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 84 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 85 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 86 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 87 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 88 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 89 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 90 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 91 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 92 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 93 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 94 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 95 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 96 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 97 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 98 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 99 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 100 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 101 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 102 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 103 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 104 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 105 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 106 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 107 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 108 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 109 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 110 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 111 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 112 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 113 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 114 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 115 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 116 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 117 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 118 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 119 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 120 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 121 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 122 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 123 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 124 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 125 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 126 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 127 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 128 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 129 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 130 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 131 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 132 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 133 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 134 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 135 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 136 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 137 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 138 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 139 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 70:
YY_RULE_SETUP
#line 140 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 141 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 72:
YY_RULE_SETUP
#line 142 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 73:
YY_RULE_SETUP
#line 143 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 74:
YY_RULE_SETUP
#line 144 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 145 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 76:
YY_RULE_SETUP
#line 146 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 77:
YY_RULE_SETUP
#line 147 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 148 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 79:
YY_RULE_SETUP
#line 149 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 80:
YY_RULE_SETUP
#line 150 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 81:
YY_RULE_SETUP
#line 151 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 82:
YY_RULE_SETUP
#line 152 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 83:
YY_RULE_SETUP
#line 153 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 84:
YY_RULE_SETUP
#line 154 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 85:
YY_RULE_SETUP
#line 155 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 86:
YY_RULE_SETUP
#line 156 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 87:
YY_RULE_SETUP
#line 157 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 88:
YY_RULE_SETUP
#line 158 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 89:
YY_RULE_SETUP
#line 159 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 90:
YY_RULE_SETUP
#line 160 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 91:
YY_RULE_SETUP
#line 161 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 92:
YY_RULE_SETUP
#line 162 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 93:
YY_RULE_SETUP
#line 163 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 94:
YY_RULE_SETUP
#line 164 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 95:
YY_RULE_SETUP
#line 166 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 96:
YY_RULE_SETUP
#line 167 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 97:
YY_RULE_SETUP
#line 168 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 98:
YY_RULE_SETUP
#line 169 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 99:
YY_RULE_SETUP
#line 170 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 100:
YY_RULE_SETUP
#line 171 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 101:
YY_RULE_SETUP
#line 172 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 102:
YY_RULE_SETUP
#line 173 "vlex.l"
{ return V_KW; }
	YY_BREAK
case 103:
YY_RULE_SETUP
#line 175 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 104:
YY_RULE_SETUP
#line 176 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 105:
YY_RULE_SETUP
#line 177 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 106:
YY_RULE_SETUP
#line 178 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 107:
YY_RULE_SETUP
#line 179 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 108:
YY_RULE_SETUP
#line 180 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 109:
YY_RULE_SETUP
#line 181 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 110:
YY_RULE_SETUP
#line 182 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 111:
YY_RULE_SETUP
#line 183 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 112:
YY_RULE_SETUP
#line 184 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 113:
YY_RULE_SETUP
#line 185 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 114:
YY_RULE_SETUP
#line 186 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 115:
YY_RULE_SETUP
#line 187 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 116:
YY_RULE_SETUP
#line 188 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 117:
YY_RULE_SETUP
#line 189 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 118:
YY_RULE_SETUP
#line 190 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 119:
YY_RULE_SETUP
#line 191 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 120:
YY_RULE_SETUP
#line 192 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 121:
YY_RULE_SETUP
#line 193 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 122:
YY_RULE_SETUP
#line 194 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 123:
YY_RULE_SETUP
#line 195 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 124:
YY_RULE_SETUP
#line 196 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 125:
YY_RULE_SETUP
#line 197 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 126:
YY_RULE_SETUP
#line 199 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 127:
YY_RULE_SETUP
#line 200 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 128:
YY_RULE_SETUP
#line 201 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 129:
YY_RULE_SETUP
#line 202 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 130:
YY_RULE_SETUP
#line 203 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 131:
YY_RULE_SETUP
#line 204 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 132:
YY_RULE_SETUP
#line 205 "vlex.l"
{ return V_KW_2005; }
	YY_BREAK
case 133:
YY_RULE_SETUP
#line 207 "vlex.l"
{ return V_MODULE; }
	YY_BREAK
case 134:
YY_RULE_SETUP
#line 208 "vlex.l"
{ return V_MODULE; }
	YY_BREAK
case 135:
YY_RULE_SETUP
#line 209 "vlex.l"
{ return V_MODULE; }
	YY_BREAK
case 136:
YY_RULE_SETUP
#line 211 "vlex.l"
{ return V_ENDMODULE; }
	YY_BREAK
case 137:
YY_RULE_SETUP
#line 212 "vlex.l"
{ return V_ENDMODULE; }
	YY_BREAK
case 138:
YY_RULE_SETUP
#line 214 "vlex.l"
{ return V_PREPROC_WS; }
	YY_BREAK
case 139:
/* rule 139 can match eol */
YY_RULE_SETUP
#line 215 "vlex.l"
{ my_yylineno++; BEGIN INITIAL; return V_WS; }
	YY_BREAK
case 140:
YY_RULE_SETUP
#line 217 "vlex.l"
{ 
							if(is_builtin_define (yytext+1, yyleng-1))
								{
								BEGIN PREPROC;
								return(V_PREPROC);
								}

							return(V_MACRO);
							}
	YY_BREAK
case 141:
YY_RULE_SETUP
#line 228 "vlex.l"
{ return V_ID; }
	YY_BREAK
case 142:
YY_RULE_SETUP
#line 229 "vlex.l"
{ return V_FUNC; }
	YY_BREAK
case 143:
YY_RULE_SETUP
#line 230 "vlex.l"
{ return V_ID; }
	YY_BREAK
case 144:
YY_RULE_SETUP
#line 231 "vlex.l"
{ return V_ID; }
	YY_BREAK
case 145:
YY_RULE_SETUP
#line 232 "vlex.l"
{ return V_PORT; }
	YY_BREAK
case 146:
YY_RULE_SETUP
#line 233 "vlex.l"
{ return V_NUMBER; }
	YY_BREAK
case 147:
YY_RULE_SETUP
#line 234 "vlex.l"
{ return V_NUMBER; }
	YY_BREAK
case 148:
YY_RULE_SETUP
#line 235 "vlex.l"
{ return V_NUMBER; }
	YY_BREAK
case 149:
YY_RULE_SETUP
#line 236 "vlex.l"
{ return V_NUMBER; }
	YY_BREAK
case 150:
YY_RULE_SETUP
#line 237 "vlex.l"
{ return V_NUMBER; }
	YY_BREAK
case 151:
YY_RULE_SETUP
#line 238 "vlex.l"
{ return V_IGNORE; }
	YY_BREAK
case 152:
YY_RULE_SETUP
#line 239 "vlex.l"
{ return V_WS; }
	YY_BREAK
case 153:
/* rule 153 can match eol */
YY_RULE_SETUP
#line 240 "vlex.l"
{ my_yylineno++; return V_WS; }
	YY_BREAK
case 154:
YY_RULE_SETUP
#line 241 "vlex.l"
{ return V_CMT; }
	YY_BREAK
case 155:
YY_RULE_SETUP
#line 242 "vlex.l"
{ return V_STRING; }
	YY_BREAK
case 156:
YY_RULE_SETUP
#line 243 "vlex.l"
{ return V_IGNORE; } 
	YY_BREAK
case 157:
YY_RULE_SETUP
#line 245 "vlex.l"
ECHO;
	YY_BREAK
#line 2094 "vlex.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(CMNT):
case YY_STATE_EOF(PREPROC):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( yywrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			yy_size_t num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				yy_size_t new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart(yyin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) yyrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 619 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 619 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 618);

		return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			yy_size_t offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart(yyin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve yytext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void yyrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer(yyin,YY_BUF_SIZE );
	}

	yy_init_buffer(YY_CURRENT_BUFFER,input_file );
	yy_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void yy_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * 
 */
    void yy_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree((void *) b->yy_ch_buf  );

	yyfree((void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	yy_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void yy_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack();

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void yypop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void yyensure_buffer_stack (void)
{
	yy_size_t num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (yyconst char * yystr )
{
    
	return yy_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (yyconst char * yybytes, yy_size_t  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	yy_size_t i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) yyalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = (yy_hold_char); \
		(yy_c_buf_p) = yytext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int yyget_lineno  (void)
{
        
    return yylineno;
}

/** Get the input stream.
 * 
 */
FILE *yyget_in  (void)
{
        return yyin;
}

/** Get the output stream.
 * 
 */
FILE *yyget_out  (void)
{
        return yyout;
}

/** Get the length of the current token.
 * 
 */
yy_size_t yyget_leng  (void)
{
        return yyleng;
}

/** Get the current token.
 * 
 */

char *yyget_text  (void)
{
        return yytext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void yyset_lineno (int  line_number )
{
    
    yylineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  in_str )
{
        yyin = in_str ;
}

void yyset_out (FILE *  out_str )
{
        yyout = out_str ;
}

int yyget_debug  (void)
{
        return yy_flex_debug;
}

void yyset_debug (int  bdebug )
{
        yy_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}

/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state();
	}

	/* Destroy the stack itself. */
	yyfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *yyrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void yyfree (void * ptr )
{
	free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 245 "vlex.l"



